# -*- coding: UTF-8 -*-

import os
from common import config, filesmodify, logger
from datetime import datetime
import time

logInfo = "ORACLE converter: "
idLine = 0                  # Номер строки по порядку
saleFiles = set()


def alignmentLeft(text, lenString, character):
    if len(text) < lenString:
        text = text.ljust(lenString, character)
    else:
        text = text[:lenString]
    return text


def alignmentRight(text, lenString, character):
    if len(text) < lenString:
        text = text.rjust(lenString, character)
    else:
        text = text[:lenString]
    return text


def fhead(file, businessDate, idShop):                          # Только 1 на файл, Обязательный
    global idLine
    idLine += 1
    line = 'FHEAD'                                              # (Y) File Type Record Descriptor    Идентифицирует тип записи
    line += alignmentRight(str(idLine), 10, '0')                # (Y) File Line Identifier           ID текущей строки файла.
    line += 'RTLG'                                              # (Y) File Type Definition           Идентифицирует файл как «Oracle Retail TLOG».
    line += datetime.now().strftime('%Y%m%d%H%M%S')             # (Y) File Create Date               Дата и время создания файла внешней  системой (YYYYMMDDHHMMSS).
    line += businessDate.strftime('%Y%m%d')                     # (Y) Business Date                  Бизнес-дата транзакции. (YYYYMMDD).
    line += alignmentLeft(str(idShop), 10, ' ')                 # (Y) Location Number                Идентификатор склада или магазина имагазина
    line += alignmentLeft('', 30, ' ')                          # (N) Reference Number               Может содержать Polling ID (ID передачи) связанного с консолидированным TLOG file или использовано для других целей 

    file.write(line + '\n')


def ftail(file, countCheck):                                    # Только 1 на файл, Обязательный
    global idLine
    idLine += 1
    line = 'FTAIL'                                              # (Y) File Type Record Descriptor    Идентификатор типа записи
    line += alignmentRight(str(idLine), 10, '0')                # (Y) File Line Identifier           ID текущей строки
    line += alignmentRight(str(idLine - 2), 10, '0')            # (Y) File Record Counter            Количество транзакций в текущем файле (только записи между file head & tail)

    file.write(line + '\n')


def thead(file, check, transactionNumber, transactionType, sum=0):  # Множество, Один на транзакцию, Обязательный для каждой транзакции
    global idLine
    idLine += 1
    transactionNumber += 1

    valueSign = ''
    value = ''
    valueFillCharacter = ' '
    refNum1 = ''
    cashCode = check['cashCode']
    tranDate = check['timeEnd']
    if transactionType == 'TOTAL':
        valueSign = 'P' if sum >= 0 else 'N'
        value = str(int(sum * 10000))
        valueFillCharacter = '0'
        refNum1 = '1000'
        cashCode = ''
        seconds = time.mktime(check['timeEnd'].timetuple()) + 1
        tranDate = datetime.fromtimestamp(seconds)

    line = 'THEAD'                                              # File Type Record Descriptor
    line += alignmentRight(str(idLine), 10, '0')                # File Line Identifier
    line += alignmentLeft(cashCode, 5, ' ')                     # Register
    line += tranDate.strftime('%Y%m%d%H%M%S')                   # Transaction Date
    line += alignmentRight(str(transactionNumber), 10, '0')     # Transaction Number
    line += alignmentLeft('', 10, ' ')                          # (N) Cashier                    Cashier identifier.
    line += alignmentLeft('', 10, ' ')                          # (N) Salesperson                Salesperson identifier.
    line += alignmentLeft(transactionType, 6, ' ')              # (Y) Transaction Type
    line += alignmentLeft('', 6, ' ')                           # (N) Sub-transaction type       Sub-transaction type. For sale, it can be employee, drive-off etc. 
    line += alignmentRight('', 10, ' ')                         # (N) Orig_tran_no               Populated only for post-void transactions. Transaction number for the original tran that will be cancelled.  
    line += alignmentLeft('', 5, ' ')                           # (N) Orig_reg_no                Populated only for post- void transactions. Register number from the original tran. 
    line += alignmentLeft('', 6, ' ')                           # (N) Reason Code                Reason entered by cashier for some transaction types. Required for Paid In and Paid out transaction types, but can also be used for voids, returns, etc. 
    line += alignmentLeft('', 10, ' ')                          # (N) Vendor Number              Supplier id for a merchandise vendor paid out transaction, partner id for an expense vendor paid out transaction. 
    line += alignmentLeft('', 30, ' ')                          # (N) Vendor Invoice Number      Invoice number for a vendor paid out transaction. 
    line += alignmentLeft('', 16, ' ')                          # (N) Payment Reference Number   The reference number of the tender used for a vendor payout. This could be the money order number, check number, etc.  
    line += alignmentLeft('', 30, ' ')                          # (N) Proof of Delivery Number   Proof of receipt number given by the vendor at the time of delivery. This field is populated for a vendor paid out transaction. 
    line += alignmentLeft(refNum1, 30, ' ')                     # (N) Reference Number 1         Number associated with a particular transaction, for example weather for a Store Conditions transaction. 
    line += alignmentLeft('', 30, ' ')                          # (N) Reference Number 2         Second generic reference number. 
    line += alignmentLeft('', 30, ' ')                          # (N) Reference Number 3         Third generic reference number. 
    line += alignmentLeft('', 30, ' ')                          # (N) Reference Number 4         Fourth generic reference number. 
    line += alignmentLeft(valueSign, 1, ' ')                    # (Y) Value Sign                 Sign of the value.
    line += alignmentRight(value, 20, valueFillCharacter)       # (Y) Value                      Value with 4 implied decimal places. Populated by the retailer for TOTAL trans, populated by Oracle Retail sales audit for SALE, RETURN trans. 
    line += alignmentRight('', 4, ' ')                          # (Y) Banner id                  Banner ID of the location
    line += alignmentRight('', 30, ' ')                         # (N) Customer order head id     Customer head order number. 
    line += alignmentLeft('', 14, ' ')                          # (N) Customer order head date   Customer order date
                                                                # new
    line += alignmentLeft('', 1, ' ')                           # (Y) Rounded Amount Sign        Значение из списка «SIGN» code_type.
    line += alignmentRight('', 20, ' ')                         # (Y) Rounded Amount             Общее (total) округление количества с 4 значащими цифрами после запятой
    line += alignmentLeft('', 1, ' ')                           # (Y) RoundedOffSign             Значение из списка «SIGN» code_type.
    line += alignmentRight('', 20, ' ')                         # (Y) RoundedOff Amount          Итоговое округление количества с 4 значащими цифрами после запятой
    line += alignmentLeft('', 10, ' ')                          # (N) Credit Promotion Id        Credit Promotional Id
    line += alignmentLeft('', 30, ' ')                          # (N) Reference Number 25        Двадцать пятый универсальный справочный номер. Расположение в fiscal_note_number. Более детальное описание в sa_reference.
    line += alignmentLeft('', 30, ' ')                          # (N) Reference Number 26        Двадцать шестой универсальный справочный номер
    line += alignmentLeft('', 30, ' ')                          # (N) Reference Number 27        Двадцать седьмой универсальный справочный номер

    file.write(line + '\n')

    return transactionNumber


def ttail(file, countPosition):                                 # 1 на запись THEAD, Обязательный
    global idLine
    idLine += 1
    line = 'TTAIL'                                              # (Y) File Type Record Descriptor    Идентификатор типа записи
    line += alignmentRight(str(idLine), 10, '0')                # (Y) File Line Identifier           ID текущей строки.
    line += alignmentRight(str(countPosition), 10, '0')         # (Y) Transaction Record Counter     Количество записей в текущей транзакции (только записи между trans head & tail)

    file.write(line + '\n')


def idisc(file, disc, accountingQuant, fromScales):
    global idLine

    if disc['discSum'] == 0:
        return False

    quantitySign = 'N' if disc['discSum'] < 0 else 'P'
    discountType = 'CMPSPL' if disc['discMode'] == 'byChange' else 'SENCIT'
    discAmount = float(disc['discSum']) / accountingQuant * 10000
    discAmount = abs(int(discAmount))
    quant = str(int(round(accountingQuant * 10000)))

    idLine += 1
    line = 'IDISC'                                              # (Y) File Type Record Descriptor    Идентификатор типа записи
    line += alignmentRight(str(idLine), 10, '0')                # (Y) File Line Identifier           ID текущей строки.
    line += alignmentLeft('1004', 6, ' ')                       # (Y) RMS Promotion Number           Тип промоакции RMS.
    line += alignmentLeft('', 10, ' ')                          # (N) Discount Reference Number      Код скидки, связанный с типом скидки (например, если Тип скидки = promotion, поле содержит код промоакции).
    line += alignmentLeft(discountType, 6, ' ')                 # (N) Discount Type                  Тип скидки в промоакции.   Это позволяет в дальнейшем, например, анализировать скидки по купонам в “In-store” промоакциях
    line += alignmentLeft('', 16, ' ')                          # (Y) Coupon Number                  Номер купона, использованного в качестве скидки.
    line += alignmentLeft('', 16, ' ')                          # (Y) Coupon Reference Number        Дополнительная информация о купоне. Обычно содержит второй штрихкод купона.
    line += alignmentLeft(quantitySign, 1, ' ')                 # (Y) Quantity Sign                  Знак количества
    line += alignmentRight(quant, 12, '0')                      # (Y) Quantity                Количество, к которому была применена скидка, 4 цифры после запятой.
    line += alignmentRight(str(discAmount), 20, '0')            # (Y) Unit Discount Amount    Сумма скидки для товара, 4 цифры после запятой.
    line += alignmentLeft('', 30, ' ')                          # (N) Reference Number 13            Код, связанный с конкретный типом транзакции науровне скидки. Таблица «sa_reference» определяет возможные  значения этого поля для каждого типа транзакций.
    line += alignmentLeft('', 30, ' ')                          # (N) Reference Number 14            Второй универсальный код для уровня скидки
    line += alignmentLeft('', 30, ' ')                          # (N) Reference Number 15            Третий универсальный код для уровня скидки
    line += alignmentLeft('', 30, ' ')                          # (N) Reference Number 16            Четвертый универсальный код для уровня скидки
    line += alignmentRight(quant, 12, '0')                      # (Y) Uom_qty                 Количество проданных товаров в заданной единице измерения, 4 знака после запятой.
    line += alignmentLeft(fromScales, 1, ' ')                   # (Y) Catchweight_ind                Указывает, является ли товар «catchweight» (полученный вес)
    line += alignmentLeft(str(disc['discCode']), 10, ' ')       # (N) Promo component                Если скидка это промоакция, это поле содержит значение компонента связанного с промоакцией (код скидки)

    file.write(line + '\n')

    return True


def titem(file, pos, countPosition, dictMeasures):
    global idLine
    countPosition += 1
    idLine += 1
    itemStatus = 'R'                                            # Статус товара в транзакции, V - аннулирован, S - продан, R -  возвращен.
    quantitySign = 'N'                                          # P если товар продан; N если товар возвращен; Противоположен исходному товару если товар аннулирован
    overrideReason = ''
    origUnitRetail = ''
    charToFillOrigUnit = ' '

    if pos['opCode'] in [50, 52, 1000, 1004]:
        itemStatus = 'S'
        quantitySign = 'P'
    accountingQuant = 1.0 if float(pos['accountingQuant']) == 0.0 else float(pos['accountingQuant'])
    fromScales = 'Y' if pos['quantMode']=='fromScales' else 'N'
    measure = dictMeasures[str(pos['measureCode'])] if str(pos['measureCode']) in dictMeasures else 'EA'

    if pos['price'] != pos['finalPrice']:                            # Если было ручное изменение цены
        overrideReason = 'CP'
        origUnitRetail = str(int(float(pos['finalPrice']) * 10000))
        charToFillOrigUnit = '0'

    quant = str(int(round(accountingQuant * 10000)))

    line = 'TITEM'                                              # File Type Record Descriptor
    line += alignmentRight(str(idLine), 10, '0')                # File Line Identifier
    line += alignmentLeft(itemStatus, 6, ' ')                   # Item Status
    line += alignmentLeft('ITEM', 6, ' ')                       # Item Type             ERR (Ошибка), GCN (№ ваучера), ITEM (Товар), NMITEM (Нет. тов.), REF (Связ.товар), TERM (Отм. записи о прекр.)
    line += alignmentLeft('ITEM', 6, ' ')                       # Item number type    
    line += alignmentLeft('', 1, ' ')                           # Format ID
    line += alignmentLeft(str(pos['inventCode']), 25, ' ')      # Item
    line += alignmentLeft('', 25, ' ')                          # Reference Item
    line += alignmentLeft('', 25, ' ')                          # Non-Merchandise Item
    line += alignmentRight('', 25, ' ')                         # Voucher
    line += alignmentRight('', 4, ' ')                          # Department
    line += alignmentRight('', 4, ' ')                          # Class
    line += alignmentRight('', 4, ' ')                          # Subclass
    line += quantitySign                                        # Quantity Sign
    line += alignmentRight(quant, 12, '0')                      # Quantity
    line += alignmentLeft(measure, 4, ' ')                      # SellingUnit of Measure              ????????????        EA

    price = alignmentRight(str(int(pos['price'] * 10000)), 20, '0') # Unit Retail.    Розничная цена товара, 4 знака после запятой.
    unit = alignmentRight(origUnitRetail, 20, charToFillOrigUnit) # Original Unit Retail

    line += unit if overrideReason == 'CP' else price 
    line += alignmentLeft(overrideReason, 6, ' ')               # Override Reason       AP - Стоимость рекламы, CP - Конкурентная цена, D - Поврежденные тов., ERR -  Ошибка, MS - Для руководителей, S - Неправильные надписи, T - Неправильный тег
    line += price if overrideReason == 'CP' else unit
    line += alignmentLeft('N', 1, ' ')                          # Taxable Indicator     Указывает, является ли товар налогооблагаемым или нет.
    line += alignmentLeft('', 8, ' ')                           # Pump                  Идентификатор топливной колонки
    line += alignmentLeft('', 30, ' ')                          # Reference Number 5    Номер связанный со специфическим товаром в транзакции, например специальный номер заказа. Таблица sa_reference table определяет что может содержать это поля для каждого типа транзакций.
    line += alignmentLeft('', 30, ' ')                          # Reference Number 6    Второй универсальный справочный номер на уровне товара
    line += alignmentLeft('', 30, ' ')                          # Reference Number 7    Третий универсальный справочный номер на уровне товара
    line += alignmentLeft('', 30, ' ')                          # Reference Number 8    Четвертый универсальный справочный номер на уровне товара
    line += alignmentLeft('N' if pos['bcodeMode']=='manual' else 'Y', 1, ' ') # Item_swiped_ind    -    Указывает, был ли товар автоматически внесен в POS систему или был введен с клавиатуры
    line += alignmentLeft('', 6, ' ')                           # Return Reason Code
    line += alignmentLeft('', 10, ' ')                          # Salesperson            Продавец, продавший товар
    line += alignmentLeft('', 8, ' ')                           # Expiration_date        Дата истечения срока действия подарочного сертификата (YYYYMMDD).
    line += alignmentLeft('N', 1, ' ')                          # Drop Ship Ind          Указывает, является ли товар частью прямой поставки (drop shipment)
    line += alignmentRight(quant, 12, '0')                      # Uom_qty         Количество товара, проданное в указанных единицах измерения, 4 знака после запятой.
    line += alignmentLeft(fromScales, 1, ' ')                   # Catchweight_ind        Указывает, является ли товар «catchweight» (полученный вес).
    line += alignmentLeft(str(pos['inventCode']), 25, ' ')      # Selling item           Идентификатор продаваемого товара.
    line += alignmentLeft('', 6, ' ')                           # Customer order line no Номер заказа покупателя
    line += alignmentLeft('', 10, ' ')                          # Media id               Идентификатор «media id» покупателя
                                                                # new
    line += alignmentRight('', 21, ' ')                         # (N) Total Igtax Amount Сумма общих итогов по позиции по операциям: продажи\возвраты с 5 значащими цифрами после запятой.
    line += alignmentLeft('', 128, ' ')                         # (N) Unique_ID          Уникальный идентификационный номер – может использоваться, как Серийный номер позиции.

    file.write(line + '\n')

    for disc in pos['discountPositions']:
        if idisc(file, disc, accountingQuant, fromScales):
            countPosition += 1
    return countPosition


def ttend(file, pos, dictValuts):
    global idLine
    tenderTypeGroup = 'CASH'
    tenderTypeID = '1000'
    if str(pos['valCode']) in dictValuts:
        tenderTypeGroup = dictValuts[str(pos['valCode'])][0].strip()
        if len(dictValuts[str(pos['valCode'])]) > 1:
            tenderTypeID = dictValuts[str(pos['valCode'])][1].strip()
    tenderSign='P'# if pos['opCode'] in [44, 70, 76, 80] else 'N'
    idLine += 1
    line = 'TTEND'                                              # (Y) File Type Record Descriptor    Идентификатор типа записи
    line += alignmentRight(str(idLine), 10, '0')                # (Y) File Line Identifier           ID текущей строки.
    line += alignmentLeft(tenderTypeGroup, 6, ' ')              # (Y) Tender Type Group              CASH - Наличные, CCARD - Кредитная карта, CHECK - Персональный чек, COUPON - Купон, DCARD - Дебет. карта, DRIVEO - Неоплаченная покупка, ERR - Ошибка, MORDER - Платежное поручение, SOCASS - Соц.помощь, TERM - Запись о прекр., VOUCH - Вауч. (под. серт./кред.).
    line += alignmentLeft(tenderTypeID, 6, ' ')                 # (Y) Tender Type ID                 Группировка низкого уровня для типов оплат.
    line += alignmentLeft(tenderSign, 1, ' ')                   # (Y) Tender Sign                    Знак суммы
    line += alignmentRight(str(int(pos['sumB'] * 10000)), 20, '0') # (Y) Tender Amount               Сумма по данному типу оплаты в транзакции, 4 знака после запятой.
    line += alignmentLeft('', 40, ' ')                          # (Y) Cc_no                          Номер кредитной карты
    line += alignmentLeft('', 16, ' ')                          # (Y) Cc_auth_no                     Код авторизации кредитной карты
    line += alignmentLeft('', 6, ' ')                           # (Y) cc authorization source        ????????????
    line += alignmentLeft('', 6, ' ')                           # (Y) cc cardholder verification     ????????????
    line += alignmentLeft('', 8, ' ')                           # (Y) cc expiration date             ???????????? (YYYYMMDD)
    line += alignmentLeft('', 6, ' ')                           # (Y) cc entry mode                  Указывает, тип считывания кредитной карты – автоматическое считывание, введен вручную.
    line += alignmentLeft('', 5, ' ')                           # (N) cc terminal id                 Номер терминала, на котором проведена транзакция.
    line += alignmentLeft('', 6, ' ')                           # (Y) cc special condition           Refer to 'CCSC' code_type for as list of valid types
    line += alignmentRight('', 25, ' ')                         # (Y) Voucher_no                     Серийный номер подарочного сертификата или кредитного ваучера.
    line += alignmentLeft('', 16, ' ')                          # (Y) Coupon Number                  Номер купона производителя, использованного в качестве оплаты
    line += alignmentLeft('', 16, ' ')                          # (Y) Coupon Reference Number        Дополнительная информация о купоне, обычно содержит второй бар-код купона.
    line += alignmentLeft('', 30, ' ')                          # (N) Cheque Account Number          Проверочный номер счета
    line += alignmentRight('', 10, ' ')                         # (N) Cheque Number                  Номер чека
    line += alignmentLeft('', 6, ' ')                           # (N) Identification Metod           Метод идентификации (Водительские права, Фотография кредитной карты)
    line += alignmentLeft('', 40, ' ')                          # (N) Identification ID              Идентификатор ID  ( Номер прав,  Фото Карта)
    line += alignmentLeft('', 3, ' ')                           # (N) Original Currency              Валюта, которой проведена оплата. Знечение в таблице CURRENCIRS для действительных кодов валюты 
    line += alignmentRight('', 20, ' ')                         # (N) Original Currency Amount       Сумма по данному типу оплаты в указанной валюте, 4 знака после запятой.    
    line += alignmentLeft('', 30, ' ')                          # (N) Reference No 9                 Номер, связанный с конкретным типом транзакции на уровне типа оплаты.
    line += alignmentLeft('', 30, ' ')                          # (N) Reference No 10                Второй универсальный справочный код для уровня Типа оплаты.
    line += alignmentLeft('', 30, ' ')                          # (N) Reference No 11                Третий универсальный справочный код для уровня Типа оплаты.
    line += alignmentLeft('', 30, ' ')                          # (N) Reference No 12                Четвертый универсальный справочный код для уровня Типа оплаты..

    file.write(line + '\n')


def getAbsolutSum(money):
    if money['opCode'] in [44, 70, 76, 80]:
        return money['sumB']
    else:
        return money['sumB'] * -1


def finish(output, check, transactionNumber, sumSale, countCheck):
    global saleFiles
    thead(output, check, transactionNumber, 'TOTAL', sumSale)
    ttail(output, 0)
    ftail(output, countCheck)
    saleFiles.add(output.name)


def rewriteHead(file, timeEnd):
    file.seek(19)
    file.write('%s' % timeEnd.strftime('%Y%m%d%H%M%S'))


def run(salesFunc, securityLogsFunc, shiftsFunc, cash,  node, workDirectory, currentSales):
    global idLine
    idLine = 0
    dictValuts = {}
    dictMeasures = {}

    valuts = node.currentConverter.valutMapper
    for val in valuts:
        dictValuts[val] = valuts[val].split(',')

    for load in "upload", "unload": 
        for k, v in node.currentConverter.measuresMapper[load].items():
            dictMeasures[v] = k

    log = logger.getUnloadLogger("Node", node.id)
    log.info(logInfo + "START. Shop №" + str(node.id))
    shopCode = ''
    for shift in shiftsFunc():
        shopCode = shift['shopcode']
        break
    output = None
    salesDate = ''
    countCheck = 0
    transactionNumber = 0
    sumSale = 0
    timeEnd = None
    for check in salesFunc():
        if check['closed'] != 2:
            if salesDate[:-6] != check['timeEnd'].strftime('%m%d%Y'):
                if output:
                    finish(output, check, transactionNumber, sumSale, countCheck)
                    countCheck = 0
                    transactionNumber = 0
                    sumSale = 0
                    rewriteHead(output, timeEnd)
                    output.close()
                    os.rename(output.name, os.path.join(os.path.dirname(output.name),'RTLOG_%s_%s.DAT' % (shopCode, timeEnd.strftime('%m%d%Y%H%M%S'))))
                salesDate = check['timeEnd'].strftime('%m%d%Y%H%M%S')
                salePath = workDirectory + '/RTLOG_%s_%s.DAT' % (shopCode, salesDate)
                output = open(salePath, 'w')
                log.info(logInfo + "file is created " + salePath)
                fhead(output, check['timeEnd'], shopCode)

            countPosition = 0
            timeEnd = check['timeEnd']

            if check['docType'] in [1, 2]:
                countCheck += 1

                transactionType = 'SALE'
                if check['docType'] == 2:
                    transactionType = 'RETURN'
                else:
                    sumSale += check['docSum']

                transactionNumber = thead(output, check, transactionNumber, transactionType)
                for position in check['inventPositions']:          
                    if position['opCode'] in [50, 52, 54, 56, 58, 60, 62, 1000, 1002, 1004, 1006]:
                        countPosition = titem(output, position, countPosition, dictMeasures)

                moneys = {}
                for money in check['moneyPositions']:
                    if money['valCode'] in moneys:
                        moneys[money['valCode']]['sumB'] += getAbsolutSum(money)
                    else:
                        money['sumB'] = getAbsolutSum(money)
                        moneys[money['valCode']] = money
                for m in moneys.values():
                    ttend(output, m, dictValuts)
                    countPosition += 1
                ttail(output, countPosition)

    if output:
        finish(output, check, transactionNumber, sumSale, countCheck)
        rewriteHead(output, timeEnd)
        output.close()
        os.rename(output.name, os.path.join(os.path.dirname(output.name),'RTLOG_%s_%s.DAT' % (shopCode, timeEnd.strftime('%m%d%Y%H%M%S'))))
        converterSalesReadyFile = open('%s/ready' % workDirectory, 'w')
        converterSalesReadyFile.close()
        salesReadyFile = open('%s/ready' % (os.path.dirname(os.path.dirname(workDirectory))), 'w')
        salesReadyFile.close()
        saleFiles.add(converterSalesReadyFile.name)
        saleFiles.add(salesReadyFile.name)
    log.info(logInfo + "FINISH")


def sendSales(cash, node, workDirectory, backupDir):
    log = logger.getUnloadLogger("Node", node.id)
    return filesmodify.safeMoveDirectory(log, workDirectory, node.currentConverter.path, backupDir=backupDir,
            rewrite=node.currentConverter.rewriteDestFile, maxCountFiles=cash.maxCountFiles, actualCountDays=cash.actualCountDays)


def sendOnlineSales(cash, node, workDirectory):
    return False, []


def getSaleFiles():
    global saleFiles
    return saleFiles
