# -*- coding: UTF-8 -*-

import datetime
import os
from collections import defaultdict
from common import filesmodify, logger, macros
from decimal import Decimal

unloadDocumetTypes = [1, 2, 3, 4, 25]
saleFiles = set()


def getValCode(payment):
    valCode = str(payment)
    if valCode == '0':
        valCode = '1'
    return valCode


def addDistributedPosition(positionQuant, positionSum, positionOperation, positionValCode, distributedPositions):
    distributedPosition = {}
    distributedPosition['quant'] = positionQuant
    distributedPosition['posSum'] = positionSum
    distributedPosition['operation'] = positionOperation
    distributedPosition['valCode'] = positionValCode
    distributedPositions.append(distributedPosition)


def getDistributedPositions(position, operation, moneyForValut):
    distributedPositions = []
    positionSum = Decimal(position['posSum']).quantize(Decimal('.01'))
    positionQuant = Decimal(position['quant']).quantize(Decimal('.001'))
    if len(moneyForValut.keys()) > 0:
        payment = sorted(moneyForValut.keys())[0]

        positionValCode = getValCode(payment)
        paymentSum = Decimal(moneyForValut[payment]).quantize(Decimal('.01'))
        if paymentSum < positionSum:
            remainPositionQuant = positionQuant
            remainPositionSum = positionSum
            while Decimal(0.00).quantize(Decimal('.01')) < remainPositionSum:
                currentPositionValCode = positionValCode
                currentPositionOperation = operation
                if positionValCode != '1':
                    currentPositionOperation += 4
                currentPositionQuant = Decimal(positionQuant * paymentSum / positionSum).quantize(Decimal('.001'))
                if remainPositionQuant < Decimal(0.001).quantize(Decimal('.001')):
                    currentPositionQuant = Decimal(0.000).quantize(Decimal('.001'))
                if remainPositionQuant < currentPositionQuant:
                    currentPositionQuant = remainPositionQuant
                if remainPositionSum == paymentSum:
                    currentPositionQuant = remainPositionQuant
                remainPositionQuant = Decimal(remainPositionQuant - currentPositionQuant).quantize(Decimal('.001'))
                if remainPositionSum < paymentSum:
                    moneyForValut[payment] = Decimal(moneyForValut[payment]).quantize(Decimal('.01')) - remainPositionSum
                    currentPositionSum = remainPositionSum
                    remainPositionSum = Decimal(0.00).quantize(Decimal('.01'))
                else:
                    remainPositionSum -= paymentSum
                    currentPositionSum = paymentSum
                    del moneyForValut[payment]
                    if len(moneyForValut.keys()) > 0:
                        payment = sorted(moneyForValut.keys())[0]
                        positionValCode = getValCode(payment)
                        paymentSum = Decimal(moneyForValut[payment]).quantize(Decimal('.01'))
                addDistributedPosition(currentPositionQuant, currentPositionSum, currentPositionOperation, currentPositionValCode, distributedPositions)
        else:
            distributedPositionOperation = operation
            if positionValCode != '1':
                distributedPositionOperation += 4
            addDistributedPosition(positionQuant, positionSum, distributedPositionOperation, positionValCode, distributedPositions)
            moneyForValut[payment] -= positionSum
            if moneyForValut[payment] == 0:
                del moneyForValut[payment]
    elif positionSum == Decimal(0.00).quantize(Decimal('.01')):
        addDistributedPosition(positionQuant, positionSum, operation, '1', distributedPositions)
    return distributedPositions


def addMoneyPosition(payment, moneyPosition, moneyForValut):
    if payment not in moneyForValut:
         moneyForValut[payment] = 0
    if moneyPosition['opCode'] in [70, 74]:
         moneyForValut[payment] += moneyPosition['sumB']
    else:
         moneyForValut[payment] -= moneyPosition['sumB']


def addDistributedDiscountPosition(discSum, positionNumber, distributedDiscountPositions):
    distributedDiscountPosition = {}
    distributedDiscountPosition['discSum'] = discSum
    distributedDiscountPosition['posNum'] = positionNumber
    distributedDiscountPositions.append(distributedDiscountPosition)


def getDistributedDiscountPositions(discSum, positionNumber, positionSum, distributedPositions):
    distributedDiscountPositions = []
    if len(distributedPositions) == 1:
        addDistributedDiscountPosition(discSum, positionNumber, distributedDiscountPositions)
    else:
        remainDiscSum = Decimal(discSum).quantize(Decimal('.01'))
        for i in range(len(distributedPositions)):
            currentDiscSum = Decimal(distributedPositions[i]['posSum'] * discSum / positionSum).quantize(Decimal('.01'))
            if i + 1 == len(distributedPositions):
                currentDiscSum = remainDiscSum
            else:
                remainDiscSum -= currentDiscSum
            addDistributedDiscountPosition(currentDiscSum, positionNumber, distributedDiscountPositions)
            positionNumber += 1
    return distributedDiscountPositions


def addDiscountPositionToWriteInFile(discIndex, discPercent, discSum, posNum, discountPositionsToWriteInFile):
    discount = {}
    discount['discIndex'] = discIndex
    discount['discPercent'] = discPercent
    discount['discSum'] = discSum
    discountPositionsToWriteInFile[posNum].append(discount)


def getDiscountPositionsToWriteInFile(discIndex, discPercent, discSum, posNum, posSum, distributedPositions, discountPositionsToWriteInFile):
    distributedDiscountPositions = getDistributedDiscountPositions(discSum, posNum, posSum, distributedPositions)
    for distributedDiscountPosition in distributedDiscountPositions:
        addDiscountPositionToWriteInFile(discIndex, discPercent, distributedDiscountPosition['discSum'], 
                                         distributedDiscountPosition['posNum'], discountPositionsToWriteInFile)


def run(salesFunc, securityLogsFunc, shiftsFunc, cash, node, workDirectory, currentSales):
    log = logger.getUnloadLogger("Node", node.id)
    log.info(u"Старт конвертирования")

    dictValuts = node.currentConverter.valutMapper;

    for v in dictValuts:
        if dictValuts[v] == '':
            dictValuts[v] = 0;

    workDirectory = macros.applyMacros(workDirectory, log, shopcode=node.id, cashcode=cash.code)

    if node.currentConverter.appendSaleFiles and not currentSales:
        if not filesmodify.simpleCopyFiles(node.currentConverter.path, workDirectory, log)[0]:
            log.info(u"Не удалось скопировать файлы продаж из целевой папки в рабочую!")
            return

    plugin = __import__("unload.converter.smag.%s" % node.currentConverter.format, fromlist='q')
    header = plugin.DataHeader(workDirectory, node.currentConverter.charset, currentSales)

    curMoneys = {}
    currests = {}
    shifts = {}
    emptySales = True
    for shift in shiftsFunc():
        emptySales = False
        timeEnd = None
        if shift['timeEnd']:
            timeEnd = shift['timeEnd']
        else:
            timeEnd = datetime.datetime.now()

        if shift['shopcode']:
            shifts[shift['shift']] = {'SHOPINDEX' : shift['shopcode'], 'DATE' : timeEnd}
        else:
            shifts[shift['shift']] = {'SHOPINDEX' : 0, 'DATE' : timeEnd}

    # CHECK
    for check in salesFunc():
        earnSumCheck = 0
        lastPosNum = 0
        if not check['docType'] in unloadDocumetTypes and check['docType'] != 13:
            continue
        docNum = int(check['docNum'])
        cashCode = int(check['cashCode'])
        shiftNumber = int(check['shift'])
        shopIndex = int(shifts[shiftNumber]['SHOPINDEX'])

        if not shiftNumber in currests:
            currests[shiftNumber] = {}
            curMoneys[shiftNumber] = {}

        if not cashCode in currests[shiftNumber]:
            currests[shiftNumber][cashCode] = {'SHOPINDEX' : shopIndex, 'DATE' : shifts[shiftNumber]['DATE'], 'SALE' : 0, 'RETURN' : 0,}
            curMoneys[shiftNumber][cashCode] = {'SHOPINDEX' : shopIndex, 'PERIODIN' : 0, 'PERIODOUT' : 0, 'BEFORE' : 0, 'AFTER' : 0}


        if check['docType'] == 13:
            curMoneys[shiftNumber][cashCode]['BEFORE'] = check['docSum']

        # valCode - код базовой валюты
        valCode = None
        payment = 0
        moneyForValut = {}
        moneyForValutForDistributedPositions = {}
        for moneyPosition in check['moneyPositions']:
            if not valCode:
                valCode = moneyPosition['valCode']

            payment = int(moneyPosition['valCode'])
            # Если оплата производилась наличными, то
            if payment == 1:
                if moneyPosition['opCode'] == 80:
                    curMoneys[shiftNumber][cashCode]['PERIODIN'] += moneyPosition['sumB']
                elif moneyPosition['opCode'] == 82:
                    curMoneys[shiftNumber][cashCode]['PERIODOUT'] += moneyPosition['sumB']

                if moneyPosition['opCode'] in [80, 70, 76]:
                    curMoneys[shiftNumber][cashCode]['AFTER'] += moneyPosition['sumB']
                else:
                    curMoneys[shiftNumber][cashCode]['AFTER'] -= moneyPosition['sumB']
                payment = 0

            if node.currentConverter.distributePaymentsForPositions:
                addMoneyPosition(payment, moneyPosition, moneyForValutForDistributedPositions)
            # если код оплаты на кассе маппируется в код оплаты на АСТУ, то подменяем код оплаты
            if str(payment) in dictValuts:
                payment = int(dictValuts.get(str(payment)))
            addMoneyPosition(payment, moneyPosition, moneyForValut)

            if int(moneyPosition['opCode']) == 70:
                currests[shiftNumber][cashCode]['SALE'] += moneyPosition['sumB']
            elif int(moneyPosition['opCode']) == 72:
                currests[shiftNumber][cashCode]['SALE'] -= moneyPosition['sumB']

            if int(moneyPosition['opCode']) == 74:
                currests[shiftNumber][cashCode]['RETURN'] += moneyPosition['sumB']
            elif int(moneyPosition['opCode']) == 76:
                currests[shiftNumber][cashCode]['RETURN'] -= moneyPosition['sumB']
        for key, value in sorted(moneyForValut.items()):
            if check['docType'] in [1,2,25]:
                header.setCashPay(shopIndex, cashCode, shiftNumber, docNum, key, value)
        discountCards = {}
        positionNumber = 1
        for position in check['inventPositions']:

            datePosition = datetime.datetime.strptime(str(position['posTime']), '%Y-%m-%d %H:%M:%S')
            replace = 0
            operation = 0
            lastPosNum = int(position['posNum'])
            inventCode = position['inventCode']
            if int(position['opCode']) in [50, 52, 1000]:
                replace = 1
                operation = 1
            elif int(position['opCode']) is 60:
                inventCode = 0
            if not node.currentConverter.distributePaymentsForPositions:
                if valCode != 1:
                    operation += 4

            if not node.currentConverter.distributePaymentsForPositions:
                positionNumber = int(position['posNum'])

            distributedPositions = []
            if node.currentConverter.distributePaymentsForPositions:
                distributedPositions = getDistributedPositions(position, operation, moneyForValutForDistributedPositions)
                for distributedPosition in distributedPositions:
                   header.setCashSail(
                                     shopIndex,
                                     cashCode,
                                     shiftNumber,
                                     docNum,
                                     positionNumber,
                                     datePosition,
                                     datePosition.strftime("%H%M"),
                                     inventCode,
                                     distributedPosition['quant'],
                                     position['price'],
                                     distributedPosition['posSum'],
                                     int(position['userCode']),
                                     position['deptCode'],
                                     replace,
                                     distributedPosition['operation'],
                                     0 if distributedPosition['valCode'] == '1' else int(distributedPosition['valCode'])
                                     )
                   positionNumber += 1
            else:
            # CASHSAIL
                header.setCashSail(
                                 shopIndex,
                                 cashCode,
                                 shiftNumber,
                                 docNum,
                                 int(position['posNum']),
                                 datePosition,
                                 datePosition.strftime("%H%M"),
                                 inventCode,
                                 position['quant'],
                                 position['price'],
                                 position['posSum'],
                                 int(position['userCode']),
                                 position['deptCode'],
                                 replace,
                                 operation,
                                 0 if valCode in (1, None) else int(valCode)
                              )

            discountPositionsToWriteInFile = defaultdict(list)
            for discountPosition in position['discountPositions']:
                discountIndex = 2
                if discountPosition['discMode'] == 'fixed':
                    if discountPosition['isPositionDiscount'] == 1:
                        discountIndex = 1
                    else:
                        discountIndex = 3
                elif discountPosition['discMode'] == 'byCard':
                    discountIndex = 4

                discSum = 0
                if discountPosition['discSum']:
                    discSum = discountPosition['discSum']

                posBaseSum = 0
                if position['baseSum']:
                    posBaseSum = position['baseSum']
                if posBaseSum == 0:
                    posBaseSum = 1

                if discountPosition['discMode'] == 'byCard':
                    if discountPosition['cardNumber']:
                        if not discountPosition['cardNumber'] in discountCards:
                            discountCards[discountPosition['cardNumber']] = 0

                        discountCards[discountPosition['cardNumber']] += discountPosition['discSum']

                # CASHDISC
                if node.currentConverter.distributePaymentsForPositions:
                    getDiscountPositionsToWriteInFile(discountIndex, float(discSum / posBaseSum) * 100, discSum,
                                                      positionNumber - len(distributedPositions), position['posSum'],
                                                      distributedPositions, discountPositionsToWriteInFile)
                else:
                    header.setCashDisc(shopIndex, cashCode, shiftNumber, docNum, lastPosNum, discountIndex,
                                     float(discSum / posBaseSum) * 100,
                                     discSum
                                     )

            if node.currentConverter.distributePaymentsForPositions:
                for posNum, discounts in discountPositionsToWriteInFile.items():
                    for discount in discounts:
                        header.setCashDisc(shopIndex, cashCode, shiftNumber, docNum, posNum, discount['discIndex'],
                                     discount['discPercent'],
                                     discount['discSum']
                                     )

        # CASHDCRD
        for cardNumber in discountCards:
            header.setCashDcrd(shopIndex, cashCode, shiftNumber, docNum, cardNumber, discountCards[cardNumber])

        log.info(u"смена №%s cashcode №%s document №%s " % (shiftNumber, str(cash.code), str(check['docNum'])))

    # CURRESTS
    header.setCurrests(currests)

    # CURMONEY
    header.setCurMoney(curMoneys)
    header.close()

    if not emptySales:
        global saleFiles
        statusFileName = "ready"
        statusDir = workDirectory
        if currentSales:
            statusFileName += "Online"
            statusDir = os.path.dirname(workDirectory)

        converterSalesReadyFile = open('%s/%s' % (statusDir, statusFileName), 'w')
        converterSalesReadyFile.close()
        salesReadyFile = open('%s/%s' % (os.path.dirname(os.path.dirname(statusDir)), statusFileName), 'w')
        salesReadyFile.close()
        saleFiles.add(converterSalesReadyFile.name)
        saleFiles.add(salesReadyFile.name)
        saleFiles |= header.getSaleFiles()

    log.info(u"Конвертирование завершено")


def sendSales(cash, node, workDirectory, backupDir):
    log = logger.getUnloadLogger("Node", node.id)
    return filesmodify.safeMoveDirectory(log, workDirectory, node.currentConverter.path, backupDir=backupDir,
            rewrite=node.currentConverter.rewriteDestFile, maxCountFiles=cash.maxCountFiles, actualCountDays=cash.actualCountDays)


def sendOnlineSales(cash, node, workDirectory):
    log = logger.getUnloadLogger("Node", node.id)
    return filesmodify.simpleCopyFiles(workDirectory, node.currentConverter.onlineSalePath, log), []


def deleteCurrentSales(node, workDirectory):
    log = logger.getUnloadLogger("Node", node.id)
    return filesmodify.removeFilesInDirectory(log, node.currentConverter.onlineSalePath) and \
           filesmodify.removeFilesInDirectory(log, workDirectory)


def getSaleFiles():
    global saleFiles
    return saleFiles
